package net.yakiboo.yakito.config;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.List;

import net.yakiboo.yakito.io.YakiToFileSystem;
import net.yakiboo.yakito.io.YakiToOutput;
import net.yakiboo.yakito.io.YakiToXML;
import net.yakiboo.yakito.io.YakiToXMLSyntaxException;
import net.yakiboo.yakito.language.YakiToLanguage;
import net.yakiboo.yakito.language.YakiToNoLanguagesException;
import net.yakiboo.yakito.utils.Stopwatch;

public class YakiToConfig {

	/**
	 * Ubicacion del archivo JAR de la aplicacion en ejecucion
	 */
	private File jarLocation = null;

	/**
	 * Listado de idiomas disponibles para la interfaz de usuario
	 */
	private List<YakiToLanguage> languages = null;

	/**
	 * Ultima ubicacion de destino
	 */
	private File lastDestiny = null;

	/**
	 * Idioma actual para la interfaz.
	 */
	private YakiToLanguage language = null;

	/**
	 * Fecha de ultima actualizacion realizada
	 */
	private long lastKnownUpdate = 0;

	/**
	 * Ultimo perfil ejecutado
	 */
	private String last_profile;

	/**
	 * Constructor general. Se encarga de inicializar los parametros principales
	 * automaticamente.
	 */
	public YakiToConfig() throws YakiToNoLanguagesException,
			YakiToXMLSyntaxException {
		this.jarLocation = YakiToFileSystem.getJarLocation();
		this.languages = YakiToLanguage.availableLanguages();
		loadLastParameters();
	}

	/**
	 * Carga los parametros de la ultima ejecucion, como el ultimo idioma
	 * utilizado, para seguir empleandolos durante la ejecucion actual.
	 */
	private void loadLastParameters() throws YakiToNoLanguagesException,
			YakiToXMLSyntaxException {

		File cfg = YakiToFileSystem.getConfigFile();
		if (cfg.exists()) {
			HashMap<String, String> map = YakiToXML.parseSimpleXMLFile(cfg);

			// Cargamos el idioma utilizado en la ultima ejecucion
			String lastLang = map.get("language");
			int index = -1;
			if (languages != null) {
				index = languages.indexOf(new YakiToLanguage(null, lastLang,
						null));
			}
			if (index > -1) {
				this.language = languages.get(index);
				YakiToOutput.message("Se ha establecido el idioma que se usó "
						+ "en la última ejecución (" + lastLang + ").");
			} else {
				throw new YakiToNoLanguagesException();
			}

			// Cargamos la fecha de la ultima actualizacion realizada
			Long last = Long.parseLong(map.get("last_update"));
			if (last == null) {
				YakiToOutput
						.error("No se ha encontrado la fecha de la ultima actualización. Revise el archivo \"YakiTo.cfg\"");
			} else {
				lastKnownUpdate = last;
				YakiToOutput.message("Última actualizacion realizada: " + last);
			}

			// Cargamos el ultimo directorio usado como destino
			String last_destiny = map.get("last_destiny");
			if (last_destiny != null) {
				File f = new File(last_destiny);
				if (f != null) {
					setLastDestiny(f);
				}
			}

			// Cargamos el ultimo perfil utilizado
			setLast_profile(map.get("last_profile"));
		} else {
			// Creamos un fichero de configuracion nuevo y volvemos a intentarlo
			this.createDefaultConfigFile(cfg);
			this.loadLastParameters();
			YakiToOutput
					.message("Parece que es la primera vez que se ejecuta YakiTo. Creando archivo de configuración...");
		}
	}

	/**
	 * Crea un archivo de configuracion nuevo, con la configuracion por defecto
	 * de YakiTo
	 * 
	 * @param cfg
	 *            Archivo a crear
	 */
	public void createDefaultConfigFile(File cfg) {
		HashMap<String, String> map = new HashMap<String, String>();
		map.put("language", "Spanish");
		map.put("last_update", "-1");
		map.put("last_destiny", System.getProperty("user.home"));
		YakiToXML.writeXMLFile(map, cfg);
	}

	/**
	 * Obtiene el idioma asociado a la configuracion
	 * 
	 * @return Idioma de la configuracion
	 */
	public YakiToLanguage getLanguage() {
		return language;
	}

	/**
	 * Obtiene la ruta de destino que se uso por ultima vez
	 * 
	 * @return Ruta de destino de los archivos convertidos
	 */
	public File getLastDestiny() {
		return lastDestiny;
	}

	/**
	 * Establece el directorio de destino de los archivos generados
	 * 
	 * @param f
	 *            Directorio de destino de los archivos convertidos
	 */
	public void setLastDestiny(File f) {
		lastDestiny = f;
	}

	/**
	 * Obtiene el directorio de almacenamiento de idiomas de la interfaz
	 * 
	 * @return Directorio donde se almacenan los idiomas de YakiTo
	 */
	public File getLangLocation() {
		return YakiToFileSystem.getLangLocation();
	}

	/**
	 * Obtiene un listado con todos los idiomas disponibles para YakiTo
	 * 
	 * @return Lista de idiomas de la interfaz
	 * @throws YakiToNoLanguagesException
	 */
	public List<YakiToLanguage> getAvailableLanguages()
			throws YakiToNoLanguagesException {
		return YakiToLanguage.availableLanguages();
	}

	/**
	 * Obtiene el identificador de la ultima actualizacion realizada
	 * 
	 * @return Identificador de actualizacion
	 */
	public long getLastKnownUpdate() {
		return lastKnownUpdate;
	}

	/**
	 * Obtiene la ruta en la que se encuentra el JAR de la aplicacion
	 * 
	 * @return Ruta de directorios
	 */
	public String getJarLocationPath() {
		return jarLocation.getAbsolutePath();
	}

	/**
	 * Crea un fichero XML de configuracion a partir de todos los atributos de
	 * la instancia.
	 * 
	 * @param f
	 *            Archivo a crear/sustituir
	 */
	public void saveConfigToXML(File f) {
		HashMap<String, String> map = new HashMap<String, String>();
		map.put("language", getLanguage().getName());
		map.put("last_update", String.valueOf(getLastKnownUpdate()));
		map.put("last_destiny", lastDestiny.getAbsolutePath());
		map.put("last_profile", this.last_profile);
		YakiToXML.writeXMLFile(map, f);
	}

	/**
	 * Guarda los cambios de configuracion
	 */
	public void saveConfigToXML() {
		saveConfigToXML(YakiToFileSystem.getConfigFile());
	}

	/**
	 * Establece el idioma de la aplicacion
	 * 
	 * @param langName
	 *            Nombre del nuevo idioma
	 */
	public void setLanguage(String langName) throws YakiToNoLanguagesException {
		YakiToLanguage aux = YakiToLanguage.getLanguageByName(langName);
		if (aux != null) {
			this.language = aux;
			saveConfigToXML();
		} else {
			YakiToOutput.error("Idioma desconocido: " + langName);
		}

	}

	/**
	 * Establece la fecha de la ultima actualizacion realizada
	 * 
	 * @param last
	 *            Nueva fecha de ultima actualizacion
	 */
	public void setLastKnownUpdate(long last) {
		lastKnownUpdate = last;
		saveConfigToXML();
	}

	/**
	 * Establece el ultimo perfil utilizado, para que en sucesivas ejecuciones
	 * se cargue dicho perfil por defecto
	 * 
	 * @param last_profile
	 */
	public void setLast_profile(String last_profile) {
		this.last_profile = last_profile;
	}

	/**
	 * Obtiene el ultimo perfil utilizado en YakiTo
	 * 
	 * @return Nombre del ultimo perfil que se utilizo
	 */
	public String getLast_profile() {
		return last_profile;
	}

}
