/**
 * YakiToXML.java
 */

package net.yakiboo.yakito.io;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import net.yakiboo.yakito.YakiToContainer;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Clase de manipulacion de archivos XML
 * 
 * @author Sergio Garcia Mondaray
 */
public class YakiToXML {

	/**
	 * Obtiene un Documento DOM manipulable, a partir de una cadena XML
	 * 
	 * @param s
	 *            Contenido XML del archivo del que queremos crear un Documento
	 * @return Documento DOM
	 */
	private static Document string2DOM(String s) {

		Document tmpX = null;
		DocumentBuilder builder = null;

		try {
			builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
			tmpX = (Document) builder.parse(new ByteArrayInputStream(s
					.getBytes()));
		} catch (javax.xml.parsers.ParserConfigurationException error) {
			YakiToOutput.error("Error crando factory String2DOM: "
					+ error.getMessage());
		} catch (org.xml.sax.SAXException error) {
			YakiToOutput.error("Error parseo SAX String2DOM: "
					+ error.getMessage());
		} catch (IOException error) {
			YakiToOutput.error("Error generando Bytes String2DOM: "
					+ error.getMessage());
		}
		return tmpX;
	}

	/**
	 * Obtiene el contenido de un fichero especificado
	 * 
	 * @param path
	 *            Fichero a leer
	 * @return Contenido del fichero
	 */
	private static String getFileAsString(File path) {
		String aux = "";
		String all = "";
		BufferedReader bf;
		FileReader fr;
		try {
			fr = new FileReader(path);
			bf = new BufferedReader(fr);
			while ((aux = bf.readLine()) != null) {
				all += aux;
			}
		} catch (java.io.IOException ex02) {
			YakiToOutput.error("Error leyendo el contenido del fichero \""
					+ path + "\"");
		}
		return all;
	}

	private static String getTagValue(Element ele, String tag) {
		String textVal = null;
		NodeList nl = ele.getElementsByTagName(tag);
		if (nl != null && nl.getLength() > 0) {
			Element el = (Element) nl.item(0);
			textVal = el.getFirstChild().getNodeValue();
		}
		return textVal;
	}

	/**
	 * Parsea un archivo XML simple (un solo nivel).
	 * 
	 * @param f
	 *            Archivo a leer
	 * @return Mapa cuyas claves son los tags del XML y cuyos contenidos son los
	 *         valores de esos tags.
	 */
	public static HashMap<String, String> parseSimpleXMLFile(File f)
			throws YakiToXMLSyntaxException {
		HashMap<String, String> map = new HashMap<String, String>();
		try {
			String fileContent = getFileAsString(f);
			Document doc = string2DOM(fileContent);
			NodeList root = doc.getElementsByTagName("root");
			Element elem = (Element) root.item(0);
			NodeList nl = elem.getElementsByTagName("*");
			Node n = null;
			for (int i = 0; i < nl.getLength(); i++) {
				n = nl.item(i);
				map.put(n.getNodeName(), n.getTextContent());
			}
			if (map.isEmpty())
				map = null;
		} catch (Exception ex) {
			throw new YakiToXMLSyntaxException(f);
		}
		return map;
	}

	/**
	 * Obtiene todos los contenedores multimedia de un fichero XML dado con el
	 * formato correcto.
	 * 
	 * @param f
	 *            archivo XML de contenedores
	 * @return Lista de contenedores multimedia
	 */
	public static List<YakiToContainer> parseContainersXMLFile(File f) {
		LinkedList<YakiToContainer> list = new LinkedList<YakiToContainer>();
		Document doc;
		doc = string2DOM(getFileAsString(f));
		int nc = 0;
		if (doc != null) {
			NodeList children = doc.getElementsByTagName("container");
			for (int i = 0; i < children.getLength(); i++) {
				YakiToContainer aux = recContainer((Element) children.item(i));
				if (aux != null) {
					list.add(aux);
					nc++;
				}
			}
			YakiToOutput.message("Generada lista de contenedores"
					+ " multimedia (encontrados " + nc + " contenedores)");
		} else {
			YakiToOutput.error("El documento de contenedores"
					+ " multimedia devuelve que openXML es NULL");
		}
		return list;
	}

	/**
	 * Obtiene el contenedor multimedia asociado a una etiqueta "container" de
	 * un XML
	 * 
	 * @param node
	 *            Nodo asociado a la etiqueta container a procesar
	 * @return Contenedor multimedia
	 */
	private static YakiToContainer recContainer(Element node) {
		YakiToContainer container = null;
		String name, extension, definition;
		if (node != null) {
			name = getTagValue(node, "name");
			extension = getTagValue(node, "extension");
			definition = getTagValue(node, "definition");
			if (name != null && extension != null && definition != null) {
				// We need a name and an extension
				if (!name.equals(" ") && !extension.equals(" ")) {
					container = new YakiToContainer(name, extension, definition);
				}
			}
		}
		return container;
	}

	/**
	 * Crea un fichero XML simple con el contenido del mapa de cadenas dado
	 * 
	 * @param map
	 *            Mapa cuyas claves son los tags, y cuyos contenidos son los
	 *            valores de esos tags
	 * @param f
	 *            Archivo a crear/sustituir
	 */
	public static void writeXMLFile(HashMap<String, String> map, File f) {
		try {
			FileWriter fw = new FileWriter(f);
			fw.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
			fw.write("<root>\n");
			for (String key : map.keySet()) {
				fw.write("\t<" + key + ">" + map.get(key) + "</" + key + ">\n");
			}
			fw.write("</root>\n");
			fw.close();
			YakiToOutput.message("Se ha creado/modificado el fichero \""
					+ f.getAbsolutePath() + "\" satisfactoriamente.");
		} catch (IOException e) {
			YakiToOutput.error("No se pudo escribir el fichero \""
					+ f.getAbsolutePath() + "\"");
		}

	}

}
